import typing
from .cache.cache_options import CacheOptions as _CacheOptions
from .task.task_options import TaskDequequeOptions as _TaskDequequeOptions


class DefaultOptions(object):
    Cache = _CacheOptions
    Task = _TaskDequequeOptions

    def __init__(
        self,
        cache: typing.Optional[_CacheOptions] = None,
        task: typing.Optional[_TaskDequequeOptions] = None,
    ) -> None:
        self.cache = cache
        self.task = task



import typing
from dacite import from_dict as dacite_from_dict, Config
from enum import Enum
import json
from dataclasses import asdict, dataclass

encoding = "utf-8"

TT = typing.TypeVar("TT")


def bytes_to_str(value: bytes) -> str:
    return value.decode(encoding)


def _order_dict(value: TT) -> TT:
    if isinstance(value, list):
        return typing.cast(TT, [_order_dict(item) for item in value])
    if isinstance(value, dict):
        result = {}
        for item_key, item_value in sorted(value.items()):
            if isinstance(value, typing.Dict):
                result[item_key] = _order_dict(item_value)
            else:
                result[item_key] = item_value
        return typing.cast(TT, result)
    return value


def to_json(value: TT, sort_dict: bool = False) -> str:
    @dataclass
    class JSON(object):
        value: TT

    json_obj = JSON(value=value)
    dict = asdict(json_obj)
    if sort_dict:
        dict = _order_dict(dict)
    return json.dumps(dict["value"], separators=(",", ":"))


def from_json(value: str, data_class: typing.Type[TT]) -> TT:
    data_class_any: typing.Any = data_class

    @dataclass
    class JSON(object):
        value: data_class_any

    value_to_json = '{{"value":{0}}}'.format(value)
    dict = json.loads(value_to_json)
    json_obj = dacite_from_dict(
        data=dict,
        data_class=JSON,
        config=Config(cast=[Enum]),
    )

    return typing.cast(TT, typing.cast(typing.Any, json_obj).value)



import typing


class CacheOptionsExpire(object):
    def __init__(
        self,
        value: int,
        is_in_ms: typing.Optional[bool] = False,
        extend_on_get: typing.Optional[bool] = False,
    ) -> None:
        self.value = value
        self.is_in_ms = is_in_ms
        self.extend_on_get = extend_on_get

    @staticmethod
    def merge(
        item1: typing.Optional["CacheOptionsExpire"],
        item2: typing.Optional["CacheOptionsExpire"],
    ) -> typing.Optional["CacheOptionsExpire"]:
        return item2


class CacheOptions(object):
    Expire = CacheOptionsExpire

    def __init__(
        self,
        expire: typing.Optional[CacheOptionsExpire] = None,
    ) -> None:
        self.expire = expire

    @staticmethod
    def merge(
        item1: typing.Optional["CacheOptions"],
        item2: typing.Optional["CacheOptions"],
    ) -> typing.Optional["CacheOptions"]:
        if item1 is None:
            return item2
        if item2 is None:
            return item1
        return CacheOptions(
            expire=CacheOptions.Expire.merge(item1.expire, item2.expire),
        )



import typing
from memorix_client_redis.json import to_json
from memorix_client_redis.memorix_base import MemorixBase


def hash_key(api: MemorixBase, id: str, key: typing.Any) -> str:
    if key is None:
        return to_json(value=[*api._namespace_name_tree, id])
    return to_json(
        value=[*api._namespace_name_tree, id, key],
        sort_dict=True,
    )



import typing


class TaskDequequeOptions(object):
    def __init__(
        self,
        take_newest: typing.Optional[bool] = False,
    ) -> None:
        self.take_newest = take_newest

    @staticmethod
    def merge(
        item1: typing.Optional["TaskDequequeOptions"],
        item2: typing.Optional["TaskDequequeOptions"],
    ) -> typing.Optional["TaskDequequeOptions"]:
        if item2 is None:
            return item1
        return item2
